"""Configures pytest and Django environment setup for Djblets.

.. important::

   Do not define plugins in this file! Plugins must be in a different
   package (such as in tests/). pytest overrides importers for plugins and
   all modules descending from that module level, which will cause extension
   importers to fail, breaking unit tests.

Version Added:
    3.0
"""

from __future__ import unicode_literals

import os
import shutil
import sys

import django
import pytest


sys.path.insert(0, os.path.join(os.path.dirname(__file__)))


@pytest.fixture(autouse=True, scope='session')
def django_setup_static(request):
    """Collect static media at session start.

    Args:
        request (object):
            The pytest request.
    """
    from django.conf import settings
    from django.core import management

    for path in (settings.MEDIA_ROOT, settings.STATIC_ROOT):
        shutil.rmtree(path, ignore_errors=True)

        if not os.path.exists(path):
            os.mkdir(path, 0o755)

    management.call_command('collectstatic',
                            verbosity=request.config.option.verbose,
                            interactive=False)


def pytest_report_header(config):
    """Return information for the report header.

    This will log the version of Django.

    Args:
        config (object):
            The pytest configuration object.

    Returns:
        list of unicode:
        The report header entries to log.
    """
    return [
        'django version: %s' % django.get_version(),
    ]



import os

DEBUG = True
PRODUCTION = False
TEMPLATE_DEBUG = DEBUG

ADMINS = (
    # ('Your Name', 'your_email@domain.com'),
)

MANAGERS = ADMINS

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'djblets_test.db',
    }
}


# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'America/Chicago'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

USE_TZ = True

# Absolute path to the directory that holds media.
# Example: "/home/media/media.lawrence.com/"
STATIC_ROOT = os.path.abspath(os.path.join(__file__, '..', 'static'))
MEDIA_ROOT = os.path.abspath(os.path.join(__file__, '..', 'media'))

MEDIA_URL = '/media/'

# URL that handles the media served from STATIC_ROOT. Make sure to use a
# trailing slash if there is a path component (optional in other cases).
# Examples: "http://media.lawrence.com", "http://example.com/media/"
STATIC_URL = '/'

# URL prefix for admin media -- CSS, JavaScript and images. Make sure to use a
# trailing slash.
# Examples: "http://foo.com/media/", "/media/".
ADMIN_MEDIA_PREFIX = '/media/'

# Make this unique, and don't share it with anybody.
SECRET_KEY = 'af=y9ydd51a0g#bevy0+p#(7ime@m#k)$4$9imoz*!rl97w0j0'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = [
    ('django.template.loaders.cached.Loader', (
        'django.template.loaders.filesystem.Loader',
        'django.template.loaders.app_directories.Loader',
    )),
]

TEMPLATE_DIRS = [
    os.path.abspath(os.path.join(os.path.dirname(__file__), 'templates')),
]

CONTEXT_PROCESSORS = []

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': TEMPLATE_DIRS,
        'OPTIONS': {
            'debug': True,
            'context_processors': CONTEXT_PROCESSORS,
            'loaders': TEMPLATE_LOADERS,
        },
    },
]

MIDDLEWARE = [
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
]

ROOT_URLCONF = 'djblets.testing.urls'


# Explicitly set TEST_RUNNER to avoid incorrect compatibility check warnings
# from Django 1.7.
TEST_RUNNER = 'djblets.testing.testrunners.TestRunner'

base_path = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                         "..", "djblets"))


NODE_PATH = os.path.join(base_path, '..', 'node_modules')
os.environ['NODE_PATH'] = NODE_PATH

PIPELINE = {
    'PIPELINE_ENABLED': True,
    'COMPILERS': [
        'djblets.pipeline.compilers.es6.ES6Compiler',
        'djblets.pipeline.compilers.less.LessCompiler',
    ],
    'CSS_COMPRESSOR': None,
    'JS_COMPRESSOR': 'pipeline.compressors.uglifyjs.UglifyJSCompressor',
    'BABEL_BINARY': os.path.join(NODE_PATH, '.bin', 'babel'),
    'BABEL_ARGUMENTS': [
        '--presets', '@babel/preset-env',
        '--plugins', ['dedent', 'django-gettext'],
        '-s', 'true',
    ],
    'LESS_BINARY': os.path.join(NODE_PATH, 'less', 'bin', 'lessc'),
    'LESS_ARGUMENTS': [
        '--no-color',
        '--source-map',
        '--js',
        '--autoprefix',
    ],
    'UGLIFYJS_BINARY': os.path.join(NODE_PATH, 'uglify-js', 'bin', 'uglifyjs'),
}


INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.staticfiles',
    'oauth2_provider',
    'pipeline',
]


STATICFILES_DIRS = (
    ('djblets', os.path.join(base_path, 'static', 'djblets')),
)

STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    'djblets.extensions.staticfiles.ExtensionFinder',
    'pipeline.finders.PipelineFinder',
)

STORAGES = {
    'default': {
        'BACKEND': 'django.core.files.storage.FileSystemStorage',
    },
    'staticfiles': {
        'BACKEND': 'djblets.pipeline.storage.PipelineStorage',
    },
}


for entry in os.listdir(base_path):
    fullpath = os.path.join(base_path, entry)

    if (os.path.isdir(fullpath) and
        os.path.exists(os.path.join(fullpath, "__init__.py"))):
        INSTALLED_APPS += ["djblets.%s" % entry]


INSTALLED_APPS += ['django_evolution']


OAUTH2_PROVIDER = {
    'DEFAULT_SCOPES': 'root:read',
    'SCOPES': {},
}



"""Python build backend for Djblets.

This is a specialization of the setuptools build backend, making the following
custom changes:

1. Including all of Djblets's dependencies as build-time dependencies.

   We execute code within Djblets as part of the build process, meaning that
   we need most (if not all) of the dependencies at build time. To play it
   safe, we simply include them all.

2. Introspecting djblets/dependencies.py for package metadata.

   Setuptools allows for dynamic dependencies, but only when including it
   via a requirements.txt-formatted file. We temporarily generate one of those
   for Setuptools when building the metadata.

   (Note that we have no other place to inject this, as pyproject.toml's
   dependencies, even if empty/not specified, override anything we could set
   anywhere else.)

3. Building media and i18n files.

   When building wheels or source distributions, we run our media-building
   scripts, ensuring they get included in the resulting files.

Version Added:
    5.2
"""

from __future__ import annotations

import os
import subprocess
import sys
from typing import Optional

from setuptools import build_meta as _build_meta

from djblets.dependencies import build_dependency_list, package_dependencies


def get_requires_for_build_editable(
    config_settings: Optional[dict] = None,
) -> list[str]:
    """Return build-time requirements for editable builds.

    This will return the standard Djblets dependencies, along with any
    pyproject-specified build-time dependencies.

    Args:
        config_settings (dict, optional):
            Configuration settings to pass to Setuptools.

    Returns:
        list of str:
        The list of build-time dependencies.
    """
    _write_dependencies()

    return (
        _get_build_dependencies() +
        _build_meta.get_requires_for_build_wheel(config_settings)
    )


def get_requires_for_build_sdist(
    config_settings: Optional[dict] = None,
) -> list[str]:
    """Return build-time requirements for source distributions.

    This will return the standard Djblets dependencies, along with any
    pyproject-specified build-time dependencies.

    Args:
        config_settings (dict, optional):
            Configuration settings to pass to Setuptools.

    Returns:
        list of str:
        The list of build-time dependencies.
    """
    _write_dependencies()

    return (
        _get_build_dependencies() +
        _build_meta.get_requires_for_build_wheel(config_settings)
    )


def get_requires_for_build_wheel(
    config_settings: Optional[dict] = None,
) -> list[str]:
    """Return build-time requirements for wheel distributions.

    This will return the standard Djblets dependencies, along with any
    pyproject-specified build-time dependencies.

    Args:
        config_settings (dict, optional):
            Configuration settings to pass to Setuptools.

    Returns:
        list of str:
        The list of build-time dependencies.
    """
    _write_dependencies()

    return (
        _get_build_dependencies() +
        _build_meta.get_requires_for_build_wheel(config_settings)
    )


def prepare_metadata_for_build_editable(
    metadata_directory: str,
    config_settings: Optional[dict] = None,
) -> str:
    """Prepare metadata for an editable build.

    This will write out Djblets's dependencies to a temporary file so that
    pyproject.toml can locate it.

    Args:
        metadata_directory (str):
            The target directory for metadata.

        config_settings (dict, optional):
            Configuration settings to pass to Setuptools.

    Returns:
        str:
        The basename for the generated ``.dist-info`` directory.
    """
    _write_dependencies()

    return _build_meta.prepare_metadata_for_build_editable(
        metadata_directory,
        config_settings)


def prepare_metadata_for_build_wheel(
    metadata_directory: str,
    config_settings: Optional[dict] = None,
) -> str:
    """Prepare metadata for a wheel distribution.

    This will write out Djblets's dependencies to a temporary file so that
    pyproject.toml can locate it.

    Args:
        metadata_directory (str):
            The target directory for metadata.

        config_settings (dict, optional):
            Configuration settings to pass to Setuptools.

    Returns:
        str:
        The basename for the generated ``.dist-info`` directory.
    """
    _write_dependencies()

    return _build_meta.prepare_metadata_for_build_wheel(
        metadata_directory,
        config_settings)


def build_editable(
    wheel_directory: str,
    config_settings: Optional[dict] = None,
    metadata_directory: Optional[str] = None,
) -> str:
    """Build an editable environment.

    This will build the static media and i18n files needed by Djblets, and
    then let Setuptools build the editable environment.

    Args:
        wheel_directory (str):
            The directory where the editable wheel will be placed.

        config_settings (dict, optional):
            Configuration settings to pass to Setuptools.

        metadata_directory (str, optional):
            The directory where metadata would be stored.

    Returns:
        str:
        The basename for the generated source distribution file.
    """
    _build_data_files(collect_static=False)

    return _build_meta.build_editable(
        wheel_directory,
        {
            'editable_mode': 'compat',
            **(config_settings or {})
        },
        metadata_directory)


def build_sdist(
    sdist_directory: str,
    config_settings: Optional[dict] = None,
) -> str:
    """Build a source distribution.

    This will build the static media and i18n files needed by Djblets, and
    then let Setuptools build the distribution.

    Args:
        sdist_directory (str):
            The directory where the source distribution will be placed.

        config_settings (dict, optional):
            Configuration settings to pass to Setuptools.

    Returns:
        str:
        The basename for the generated source distribution file.
    """
    _build_data_files()

    return _build_meta.build_sdist(sdist_directory,
                                   config_settings)


def build_wheel(
    wheel_directory: str,
    config_settings: Optional[dict] = None,
    metadata_directory: Optional[str] = None,
) -> str:
    """Build a wheel.

    This will build the static media and i18n files needed by Djblets, and
    then let Setuptools build the distribution.

    Args:
        wheel_directory (str):
            The directory where the wheel will be placed.

        config_settings (dict, optional):
            Configuration settings to pass to Setuptools.

        metadata_directory (str, optional):
            The directory where metadata would be stored.

    Returns:
        str:
        The basename for the generated wheel file.
    """
    _build_data_files()

    return _build_meta.build_wheel(wheel_directory,
                                   config_settings,
                                   metadata_directory)


def _get_build_dependencies() -> list[str]:
    # For a build, we need django-evolution. Rather than hard-coding the
    # dependency twice, pull this from dev-requirements.txt.
    with open('dev-requirements.txt', 'r') as fp:
        dev_deps = [
            dep.strip()
            for dep in fp
            if dep.startswith('django_evolution')
        ]

    return build_dependency_list(package_dependencies) + dev_deps


def _write_dependencies() -> None:
    """Temporarily write dependencies to a file.

    This will write to :file:`package-requirements.txt`, so that
    :file:`pyproject.toml` can reference it.

    Context:
        The file will exist until the context closes.
    """
    with open('package-requirements.txt', 'w') as fp:
        fp.write('%s\n'
                 % '\n'.join(build_dependency_list(package_dependencies)))


def _build_data_files(
    *,
    collect_static: bool = True,
) -> None:
    """Build static media and i18n data files.

    Args:
        collect_static (bool, optional):
            Whether to run a ``collectstatic`` operation to build media
            files.

            If ``False``, support for building static media will still be
            installed.

    Raises:
        RuntimeError:
            There was an error building the media or i18n files.
    """
    media_env: dict[str, str] = os.environ.copy()

    if not collect_static:
        media_env['RUN_COLLECT_STATIC'] = '0'

    # Build the static media.
    retcode = subprocess.call(
        [
            sys.executable,
            os.path.join('contrib', 'internal', 'build-media.py'),
        ],
        env=media_env)

    if retcode != 0:
        raise RuntimeError('Failed to build media files')

    # Build the i18n files.
    retcode = subprocess.call([
        sys.executable,
        os.path.join('contrib', 'internal', 'build-i18n.py'),
    ])

    if retcode != 0:
        raise RuntimeError('Failed to build i18n files')
